@@ -2,19 +2,18 @@ from abc import ABCMeta, abstractmethod
 from contextlib import suppress
 from datetime import timedelta
 from enum import Enum
+from functools import lru_cache, wraps
 from pathlib import Path
-from time import time
-from typing import TYPE_CHECKING, Any, Dict, Optional, Union
+from time import monotonic_ns, time
+from typing import Any, Callable, TypedDict
 
 import pandas as pd
 
-if TYPE_CHECKING:
-    from typing_extensions import TypedDict
 
-    class InMemoryCached(TypedDict):
-        value: pd.DataFrame
-        mtime: float
-        created_at: float
+class InMemoryCached(TypedDict):
+    value: pd.DataFrame
+    mtime: float
+    created_at: float
 
 
 class CacheEnum(str, Enum):
@@ -34,9 +33,9 @@ class Cache(metaclass=ABCMeta):
     @staticmethod
     def should_invalidate(
         *,
-        mtime: Optional[float] = None,
+        mtime: float | None = None,
         cached_mtime: float,
-        expire: Optional[timedelta] = None,
+        expire: timedelta | None = None,
         cached_created_at: float,
     ) -> bool:
         now = time()
@@ -50,12 +49,12 @@ class Cache(metaclass=ABCMeta):
 
     @abstractmethod
     def get(
-        self, key: str, mtime: Optional[float] = None, expire: Optional[timedelta] = None
+        self, key: str, mtime: float | None = None, expire: timedelta | None = None
     ) -> pd.DataFrame:
         """get a cached value"""
 
     @abstractmethod
-    def set(self, key: str, value: pd.DataFrame, mtime: Optional[float] = None) -> None:
+    def set(self, key: str, value: pd.DataFrame, mtime: float | None = None) -> None:
         """set a cached value"""
 
     @abstractmethod
@@ -65,10 +64,10 @@ class Cache(metaclass=ABCMeta):
 
 class InMemoryCache(Cache):
     def __init__(self) -> None:
-        self._cache: Dict[str, "InMemoryCached"] = {}
+        self._cache: dict[str, InMemoryCached] = {}
 
     def get(
-        self, key: str, mtime: Optional[float] = None, expire: Optional[timedelta] = None
+        self, key: str, mtime: float | None = None, expire: timedelta | None = None
     ) -> pd.DataFrame:
         cached = self._cache[key]
         if self.should_invalidate(
@@ -80,7 +79,7 @@ class InMemoryCache(Cache):
             self.delete(key)
         return self._cache[key]["value"]
 
-    def set(self, key: str, value: pd.DataFrame, mtime: Optional[float] = None) -> None:
+    def set(self, key: str, value: pd.DataFrame, mtime: float | None = None) -> None:
         mtime = mtime or time()
         self._cache[key] = {"value": value, "mtime": mtime, "created_at": time()}
 
@@ -92,7 +91,7 @@ class InMemoryCache(Cache):
 class HDFCache(Cache):
     META_DF_KEY = "__meta__"
 
-    def __init__(self, cache_dir: Union[str, Path]) -> None:
+    def __init__(self, cache_dir: str | Path) -> None:
         self.cache_dir = Path(cache_dir).resolve()
 
     def get_metadata(self) -> pd.DataFrame:
@@ -118,7 +117,7 @@ class HDFCache(Cache):
         df.to_hdf(self.cache_dir / self.META_DF_KEY, self.META_DF_KEY, mode="w")
 
     def get(
-        self, key: str, mtime: Optional[float] = None, expire: Optional[timedelta] = None
+        self, key: str, mtime: float | None = None, expire: timedelta | None = None
     ) -> pd.DataFrame:
         metadata = self.get_metadata()
         try:
@@ -140,7 +139,7 @@ class HDFCache(Cache):
         except FileNotFoundError:
             raise KeyError(key)
 
-    def set(self, key: str, value: pd.DataFrame, mtime: Optional[float] = None) -> None:
+    def set(self, key: str, value: pd.DataFrame, mtime: float | None = None) -> None:
         mtime = mtime or time()
         infos = {"key": key, "mtime": mtime, "created_at": time()}
         metadata = self.get_metadata()
@@ -160,3 +159,37 @@ class HDFCache(Cache):
         self.set_metadata(metadata)
         with suppress(FileNotFoundError):
             (self.cache_dir / key).unlink()
+
+
+# taken from https://gist.github.com/Morreski/c1d08a3afa4040815eafd3891e16b945
+def timed_lru_cache(
+    _func: Any = None, *, seconds: int = 600, maxsize: int = 128, typed: bool = False
+) -> Callable[..., Any]:
+    """Extension of functools lru_cache with a timeout
+    Parameters:
+    seconds (int): Timeout in seconds to clear the WHOLE cache, default = 10 minutes
+    maxsize (int): Maximum Size of the Cache
+    typed (bool): Same value of different type will be a different entry
+    """
+
+    def wrapper_cache(f: Any) -> Callable[..., Any]:
+        f = lru_cache(maxsize=maxsize, typed=typed)(f)
+        f.delta = seconds * 10**9
+        f.expiration = monotonic_ns() + f.delta
+
+        @wraps(f)
+        def wrapped_f(*args: Any, **kwargs: Any) -> Any:
+            if monotonic_ns() >= f.expiration:  # pragma: no cover
+                f.cache_clear()
+                f.expiration = monotonic_ns() + f.delta
+            return f(*args, **kwargs)
+
+        wrapped_f.cache_info = f.cache_info  # type: ignore [attr-defined]
+        wrapped_f.cache_clear = f.cache_clear  # type: ignore [attr-defined]
+        return wrapped_f
+
+    # To allow decorator to be used without arguments
+    if _func is None:
+        return wrapper_cache
+    else:  # pragma: no cover
+        return wrapper_cache(_func)
