@@ -1,5 +1,6 @@
 import ftplib
 import logging
+import os
 import re
 import socket
 import ssl
@@ -10,33 +11,23 @@ from functools import partial
 from ipaddress import ip_address
 from os.path import basename, join
 from time import sleep
-from typing import (
-    IO,
-    Any,
-    Callable,
-    ContextManager,
-    Dict,
-    Generator,
-    List,
-    Optional,
-    Tuple,
-    Union,
-    cast,
-)
+from typing import IO, Any, Callable, ContextManager, Generator, cast
 from urllib.parse import ParseResult, quote, unquote, urlparse
 
 import paramiko
 
 FTP_SCHEMES = ["ftp", "ftps", "sftp"]
+_DEFAULT_MAX_TIMEOUT_SECONDS = 30
+_DEFAULT_MAX_RETRY = 7
 
-FTPClient = Union[ftplib.FTP, paramiko.SFTPClient]
+FTPClient = ftplib.FTP | paramiko.SFTPClient
 
 
 class FTPS(ftplib.FTP_TLS):
     ssl_version = ssl.PROTOCOL_TLSv1_2
 
     def connect(  # type: ignore[override]
-        self, host: str, port: Optional[int], timeout: int = 60
+        self, host: str, port: int | None, timeout: int = 60
     ) -> str:
         self.host = host
         self.port = port or 990
@@ -58,8 +49,8 @@ class FTPS(ftplib.FTP_TLS):
         return self.welcome
 
     def ntransfercmd(  # type: ignore[override]
-        self, cmd: str, rest: Optional[str] = None
-    ) -> Tuple[socket.socket, int]:
+        self, cmd: str, rest: str | None = None
+    ) -> tuple[socket.socket, int]:
         # override ntransfercmd so it reuses the sock session, to prevent SSLEOFError.
         # cf. https://stackoverflow.com/questions/40536061/ssleoferror-on-ftps-using-python-ftplib
         conn, size = ftplib.FTP.ntransfercmd(self, cmd, rest)
@@ -67,9 +58,9 @@ class FTPS(ftplib.FTP_TLS):
             conn = self.context.wrap_socket(
                 conn, server_hostname=self.host, session=self.sock.session  # type: ignore[union-attr]
             )  # this is the fix
-        return conn, size
+        return conn, size  # type:ignore[return-value] # size could be None here
 
-    def makepasv(self) -> Tuple[str, int]:
+    def makepasv(self) -> tuple[str, int]:
         # override makepasv so it rewrites the dst address if the server gave a broken one.
         # Inspired by:
         # https://github.com/lavv17/lftp/blob/d67fc14d085849a6b0418bb3e912fea2e94c18d1/src/ftpclass.cc#L774
@@ -82,10 +73,12 @@ class FTPS(ftplib.FTP_TLS):
 
 
 @contextmanager
-def ftps_client(params: ParseResult) -> Generator[Tuple[FTPS, str], None, None]:
+def ftps_client(params: ParseResult) -> Generator[tuple[FTPS, str], None, None]:
     ftps = FTPS()
     try:
-        ftps.connect(host=params.hostname or "", port=params.port, timeout=3)
+        ftps.connect(
+            host=params.hostname or "", port=params.port, timeout=_DEFAULT_MAX_TIMEOUT_SECONDS
+        )
         try:
             ftps.prot_p()
             ftps.login(user=params.username or "", passwd=params.password or "")
@@ -105,11 +98,11 @@ def ftps_client(params: ParseResult) -> Generator[Tuple[FTPS, str], None, None]:
 
 
 @contextmanager
-def ftp_client(params: ParseResult) -> Generator[Tuple[ftplib.FTP, str], None, None]:
+def ftp_client(params: ParseResult) -> Generator[tuple[ftplib.FTP, str], None, None]:
     port = params.port or 21
     ftp = ftplib.FTP()
     try:
-        ftp.connect(host=params.hostname or "", port=port, timeout=3)
+        ftp.connect(host=params.hostname or "", port=port, timeout=_DEFAULT_MAX_TIMEOUT_SECONDS)
         ftp.login(user=params.username or "", passwd=params.password or "")
         yield ftp, params.path
 
@@ -119,7 +112,7 @@ def ftp_client(params: ParseResult) -> Generator[Tuple[ftplib.FTP, str], None, N
 
 
 @contextmanager
-def sftp_client(params: ParseResult) -> Generator[Tuple[paramiko.SFTPClient, str], None, None]:
+def sftp_client(params: ParseResult) -> Generator[tuple[paramiko.SFTPClient, str], None, None]:
     port = params.port or 22
     ssh_client = paramiko.SSHClient()
     try:
@@ -129,13 +122,16 @@ def sftp_client(params: ParseResult) -> Generator[Tuple[paramiko.SFTPClient, str
             username=params.username,
             password=params.password,
             port=port,
-            timeout=3,
+            timeout=_DEFAULT_MAX_TIMEOUT_SECONDS,
         )
         sftp = ssh_client.open_sftp()
         yield sftp, params.path
 
     finally:
-        ssh_client.close()
+        # In cae of Exception, we don't want to raise it
+        with suppress(AttributeError):
+            logging.getLogger(__name__).warning("Unable to close the Connection the SSHConnection.")
+            ssh_client.close()
 
 
 def _urlparse(url: str) -> ParseResult:
@@ -149,10 +145,10 @@ def _urlparse(url: str) -> ParseResult:
     return ParseResult(*[unquote(param) for param in url_params])
 
 
-def client(url: str) -> ContextManager[Tuple[FTPClient, str]]:
+def client(url: str) -> ContextManager[tuple[FTPClient, str]]:
     parse_result = _urlparse(url)
-    ftp_client_mapping: Dict[
-        str, Callable[[ParseResult], ContextManager[Tuple[FTPClient, str]]]
+    ftp_client_mapping: dict[
+        str, Callable[[ParseResult], ContextManager[tuple[FTPClient, str]]]
     ] = {
         "ftp": ftp_client,
         "ftps": ftps_client,
@@ -189,17 +185,27 @@ def _open(url: str) -> IO[bytes]:
     return ret
 
 
-def ftp_open(url: str, retry: int = 4) -> IO[bytes]:  # type: ignore
+def ftp_open(url: str, retry: int = _DEFAULT_MAX_RETRY) -> IO[bytes]:  # type: ignore
     for i in range(1, retry + 1):
         try:
             return _open(url)
-        except (AttributeError, OSError, ftplib.error_temp) as e:
+        except (AttributeError, OSError, ftplib.error_temp, paramiko.SSHException) as e:
+            log = logging.getLogger(__name__)
+
+            # FileNotFoundError inherits from OSError
+            # We need to log that we're not seeing the specified file
+            if isinstance(e, FileNotFoundError):  # pragma: no cover
+                log.warning(
+                    f"File '{os.path.basename(url)}' not available inside : "
+                    f"'{os.path.dirname(urlparse(url).path)}' !"
+                )
+
             sleep_time = 2 * i**2
             logging.getLogger(__name__).warning(f"Retry #{i}: Sleeping {sleep_time}s because {e}")
             sleep(sleep_time)
 
 
-def _get_all_files(c: FTPClient, path: str) -> List[str]:
+def _get_all_files(c: FTPClient, path: str) -> list[str]:
     try:
         # retry_pasv returns path + the file
         return [basename(x) for x in retry_pasv(cast(ftplib.FTP, c), "nlst", path)]
@@ -209,12 +215,12 @@ def _get_all_files(c: FTPClient, path: str) -> List[str]:
         return cast(paramiko.SFTPClient, c).listdir(path)
 
 
-def ftp_listdir(url: str) -> List[str]:
+def ftp_listdir(url: str) -> list[str]:
     with client(url) as (c, path):
         return _get_all_files(c, path)
 
 
-def _get_mtime(c: FTPClient, path: str) -> Optional[int]:
+def _get_mtime(c: FTPClient, path: str) -> int | None:
     """Returns timestamp of last modification"""
     try:
         mdtm = cast(ftplib.FTP, c).sendcmd("MDTM " + path)
@@ -233,12 +239,12 @@ def _get_mtime(c: FTPClient, path: str) -> Optional[int]:
         return None
 
 
-def ftp_mtime(url: str) -> Optional[int]:
+def ftp_mtime(url: str) -> int | None:
     with client(url) as (cl_ftp, path):
         return _get_mtime(cl_ftp, path)
 
 
-def dir_mtimes(url: str) -> Dict[str, Optional[int]]:
+def dir_mtimes(url: str) -> dict[str, int | None]:
     with client(url) as (cl_ftp, path):
         mtimes_dir = {}
         all_files = _get_all_files(cl_ftp, path)
