@@ -1,5 +1,6 @@
 from .csv import csv_meta, read_csv
 from .excel import excel_meta, read_excel
+from .geodata import read_geo_data
 from .json import read_json
 from .xml import read_xml
 
@@ -14,4 +15,6 @@ __all__ = (
     "read_json",
     # XML
     "read_xml",
+    # GEOJSON
+    "read_geo_data",
 )
